#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# IMPORTS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

from src.thirdparty.code import *;

from models.generated.commands import *;
from src.core.calls import *;
from src.setup import config;
from src.models.random_walk import *;
from src.algorithms.random_walk import *;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# EXPORTS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

__all__ = [
    'endpoint_random_walk',
];

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ENDPOINT
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@run_safely()
def endpoint_random_walk(command: CommandRandomWalk) -> Result[CallResult, CallError]:
    landscape = Landscape(
        values = command.landscape.values,
        metric = command.landscape.neighbourhoods.metric,
    );
    match command.algorithm:
        case EnumWalkMode.adaptive:
            result = adaptive_walk_algorithm(
                landscape = landscape,
                r = command.landscape.neighbourhoods.radius,
                optimise = command.optimise,
                verbose = config.OPTIONS.random_walk.verbose
            );
        case EnumWalkMode.gradient:
            result = gradient_walk_algorithm(
                landscape = landscape,
                r = command.landscape.neighbourhoods.radius,
                optimise = command.optimise,
                verbose = config.OPTIONS.random_walk.verbose
            );
        case EnumWalkMode.metropolis:
            result = metropolis_walk_algorithm(
                landscape = landscape,
                r = command.landscape.neighbourhoods.radius,
                annealing = command.annealing,
                optimise = command.optimise,
                verbose = config.OPTIONS.random_walk.verbose
            );
        case _ as alg:
            raise Exception(f'No algorithm implemented for {alg.value}.');
    return Ok(CallResult(action_taken=True, message=result));
